/**************************************************************************
*  Copyright (c) 2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  19.10.2018  mifi  First Version for the TWR-K65F180M board.
**************************************************************************/
#define __MAIN_C__

/*
 * Based on the following example:
 * SDK_2.4.1_TWR-K65F180M\boards\twrk65f180m\driver_examples\sdramc
 */

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdint.h>
#include "fsl_sdram.h"
#include "pin_mux.h"
#include "clock_config.h"

/*=======================================================================*/
/*  DEFINE: Prototypes                                                   */
/*=======================================================================*/

/*=======================================================================*/
/*  DEFINE: All Structures and Common Constants                          */
/*=======================================================================*/

#define EXAMPLE_SDRAMC        SDRAM
#define SDRAM_START_ADDRESS   (0x70000000U)
#define BUS_CLK_FREQ          CLOCK_GetFreq(kCLOCK_FlexBusClk)

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  DEFINE: Definition of all local Data                                 */
/*=======================================================================*/

/*=======================================================================*/
/*  DEFINE: All code exported                                            */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*************************************************************************/
int main (void)
{
   uint32_t clockSrc;

   /* Hardware initialize. */
   uint32_t soptReg;
   uint32_t fbReg;
   BOARD_InitPins();
   BOARD_BootClockRUN();
  
   SystemCoreClockUpdate();  

   /* Set clock out to flexbus CLKOUT. */
   CLOCK_SetClkOutClock(0);

   /* Sets the Flexbus security level*/
   soptReg = SIM->SOPT2 & ~SIM_SOPT2_FBSL_MASK;
   SIM->SOPT2 = soptReg | SIM_SOPT2_FBSL(3);

   /* Enable the FB_BE_xx_yy signal in Flexbus */
   CLOCK_EnableClock(kCLOCK_Flexbus0);

   fbReg = FB->CSPMCR & ~FB_CSPMCR_GROUP2_MASK;
   FB->CSPMCR = fbReg | FB_CSPMCR_GROUP2(2);
   fbReg = FB->CSPMCR & ~FB_CSPMCR_GROUP3_MASK;
   FB->CSPMCR = fbReg | FB_CSPMCR_GROUP3(2);
   fbReg = FB->CSPMCR & ~FB_CSPMCR_GROUP4_MASK;
   FB->CSPMCR = fbReg | FB_CSPMCR_GROUP4(2);
   fbReg = FB->CSPMCR & ~FB_CSPMCR_GROUP5_MASK;
   FB->CSPMCR = fbReg | FB_CSPMCR_GROUP5(2);
   
   /* SDRAM initialize. */
   clockSrc = BUS_CLK_FREQ;
   SDRAM_Init(EXAMPLE_SDRAMC, SDRAM_START_ADDRESS, clockSrc);
  
   __asm("bkpt 0x70");
   
   return(0);
} /* main */

/*** EOF ***/
